/*******************************************************************************
Copyright © 2024 Sebastian Roith

Licensed under CC BY 4.0. To view a copy of this license, visit http://creativecommons.org/licenses/by/4.0/
*******************************************************************************/

(function(window) {

const TABLES = [
    { "name": "WEITSPRUNG", "primary": ["name", "sprungweite"], "table": [
        ["name", "versuch", "sprungweite"],
        ["Antonio", 1, 4.15],
        ["Berta",   1, 0],
        ["Cemal",   1, 3.53],
        ["Dora",    1, 2],
        ["Elena",   1, 3.64],
        ["Antonio", 2, 0],
        ["Cemal",   2, 4],
        ["Dora",    2, 3.42],
        ["Elena",   2, 0],
        ["Berta",   2, 3.06],
        ["Berta",   3, 3.24],
        ["Antonio", 3, 0],
        ["Elena",   3, 4.16],
        ["Cemal",   3, 3.81],
        ["Dora",    3, 0]
    ] }
];

const LEVELS = [
    {
        "task": "<p>In der Tabelle <code>WEITSPRUNG</code> sind die Ergebnisse beim Weitspringen in Metern eingetragen. Alle hatten drei Versuche. Es kommt schnell mal vor, dass jemand übertritt. Dann wird der Versuch als Sprungweite als 0 m gewertet.</p><p>Mache dich zunächst mit der Datenbanktabelle vertraut. Schreibe dazu zuerst eine Abfrage, die dir eine der Datenbanktabelle möglichst ähnliche Ergebnistabelle liefert. Führe sie aus und schaue dir die Ergebnistabelle genau an. Beantworte dann einige Fragen.</p>",
        "tables": TABLES,
        "locked": true,
        "checkCode": [
        ],
        "checkTermination": [
            { "type": "sameResult", "value": "SELECT * FROM WEITSPRUNG", "fine": 1, "msg": "Die Ergebnistabelle dieser Abfrage entspricht nicht bestmöglich der Datenbanktabelle…" },
            { "type": "accept", "unlock": true }
        ],
        "questionWrongFine": 1,
        "question": {
            "html": "Fülle die Lücken richtig aus:",
            "choices": [
                { "selectalways": true, "html": "Die größte Sprungweite hat ", "type": "text", "unit": " geschafft.", "accept": "Elena", "correct": true },
                { "selectalways": true, "html": "Die kürzeste Sprungweite bei einem gültigen Versuch war ", "type": "number", "min": 0, "max": 10, "step": 0.01, "accept": "2", "unit": " m.", "correct": true },
                { "selectalways": true, "html": "Am häufigsten übertreten hat ", "type": "text", "unit": ".", "accept": "Antonio", "correct": true },
                { "selectalways": true, "html": "Nur ", "type": "text", "unit": " hat bei keinem Versuch übertreten.", "accept": "Cemal", "correct": true }
            ]
        }
    },
    {
        "task": "<p>Ganz schön mühsam, die Daten selbst auszuwerten, oder? Und das war ja eine echt kurze Tabelle. Du lernst jetzt, wie du diese Arbeit (zumindest teilweise) dem Datenbankmanagementsystem übertragen kannst.</p><p>Schaue dir dazu zuerst diese SQL-Abfrage an (zunächst ohne sie auszuprobieren!) und beantworte die Frage.</p>",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE name = 'Berta'",
        "readonly": true,
        "tables": TABLES,
        "locked": false,
        "checkCode": [
            { "type": "accept", "accept": false, "fine": 1, "msg": "Du sollst die Abfrage noch nicht ausprobieren!" }
        ],
        "checkTermination": [],
        "questionWrongFine": 1,
        "question": {
            "html": "Worin unterscheidet sich diese Abfrage von bisherigen SQL-Abfragen?",
            "choices": [
                { "html": "Sie enthält ein neues Schlüsselwort, nämlich ", "type": "text", "casesensitive": false, "accept": "WHERE", "correct": true },
                { "html": "Die Reihenfolge von Tabellenname und den in die Ergebnistabelle zu übernehmenden Attributen ist vertauscht" },
                { "html": "Die Abfrage enthält eine Gleichung", "correct": true },
                { "html": "Eine zusätzliches, nicht im Tabellenschema enthaltenes Attribut wird angegeben, nämlich ", "type": "text", "response": "„WHERE“ und „'Berta'“ können kein Attributname sein, da sie erst nach „FROM“ stehen." }
            ]
        }
    },
    {
        "task": "Probiere nun diese neue SQL-Abfrage aus.",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE name = 'Berta'",
        "readonly": true,
        "tables": TABLES,
        "locked": true,
        "checkCode": [
        ],
        "checkTermination": [
            { "type": "accept", "unlock": true }
        ],
        "questionWrongFine": 1,
        "question": {
            "html": "Was bewirkt offenbar <code>WHERE name = 'Berta'</code> in der Abfrage?",
            "choices": [
                { "html": "Nur Datensätze, die diese Bedingung erfüllen, stehen in der Ergebnistabelle", "correct": true },
                { "html": "Nur Attribute, die diese Bedingung erfüllen, stehen in der Ergebnistabelle" },
                { "html": "In jedem Datensatz wird ein Attributwert entsprechend der Gleichung abgeändert" },
                { "html": "In jedem Attribut wird der Wert entsprechend der Gleichung abgeändert" }
            ]
        }
    },
    {
        "task": "<p>Nach dem Schlüsselwort <code>WHERE</code> (welches auf den Tabellennamen folgt) kannst du eine sogenannte <em>Auswahlbedingung</em> schreiben. Nur Datensätze, die die Auswahlbedingung erfüllen, werden angezeigt. Die Auswahlbedingung kann beispielsweise eine Gleichung sein. Beachte, dass du feste Zeichenketten in einfache Anführungszeichen schreiben musst.</p><p>Ändere nun die Abfrage so ab, dass statt Bertas Cemals Datensätze gezeigt werden. Führe sie dann aus.</p>",
        "keepCode": true,
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE name = 'Berta'",
        "tables": TABLES,
        "locked": true,
        "checkCode": [
            { "type": "keyword", "value": "as", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AS“ nicht erlaubt…" },
            { "type": "keyword", "value": "and", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AND“ nicht erlaubt…" },
            { "type": "keyword", "value": "or", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „OR“ nicht erlaubt…" },
            { "type": "keyword", "value": "not", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „NOT“ nicht erlaubt…" },
            { "type": "keyword", "value": "order", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „ORDER BY“ nicht erlaubt…" }
        ],
        "checkTermination": [
            { "type": "attributeCount", "value": 3, "fine": 1, "msg": "Die Ergebnistabelle soll alle Attribute enthalten" },
            { "type": "attributeName", "index": 0, "value": "name", "fine": 1, "msg": "Die Attribute und ihre Reihenfolge sollen bei der Datenbanktabelle und der Ergebnistabelle gleich sein" },
            { "type": "attributeName", "index": 1, "value": "versuch", "fine": 1, "msg": "Die Attribute und ihre Reihenfolge sollen bei der Datenbanktabelle und der Ergebnistabelle gleich sein" },
            { "type": "attributeName", "index": 2, "value": "sprungweite", "fine": 1, "msg": "Die Attribute und ihre Reihenfolge sollen bei der Datenbanktabelle und der Ergebnistabelle gleich sein" },
            { "type": "sameResult", "value": "SELECT * FROM WEITSPRUNG WHERE name = 'Cemal'", "fine": 1 },
            { "type": "accept", "unlock": true }
        ]
    },
    {
        "task": "<p>Natürlich ist keinesfalls nötig, dass das Attribut <code>name</code> in der Auswahlbedingung vorkommt. Du kannst beliebige Gleichungen formulieren.</p><p>Versuche das gleich und formuliere eine Abfrage, die dir nur alle Datensätze des zweiten Versuchs liefert.</p>",
        "keepCode": true,
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE name = 'Cemal'",
        "tables": TABLES,
        "locked": true,
        "checkCode": [
            { "type": "compareQuery", "value": "'2'", "substring": true, "inverted": true, "fine": 1, "msg": "Anführungszeichen brauchst du nur bei Zeichenketten, nicht bei Zahlen!" },
            { "type": "keyword", "value": "as", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AS“ nicht erlaubt…" },
            { "type": "keyword", "value": "and", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AND“ nicht erlaubt…" },
            { "type": "keyword", "value": "or", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „OR“ nicht erlaubt…" },
            { "type": "keyword", "value": "not", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „NOT“ nicht erlaubt…" },
            { "type": "keyword", "value": "order", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „ORDER BY“ nicht erlaubt…" }
        ],
        "checkTermination": [
            { "type": "attributeCount", "value": 3, "fine": 1, "msg": "Die Ergebnistabelle soll alle Attribute enthalten" },
            { "type": "attributeName", "index": 0, "value": "name", "fine": 1, "msg": "Die Attribute und ihre Reihenfolge sollen bei der Datenbanktabelle und der Ergebnistabelle gleich sein" },
            { "type": "attributeName", "index": 1, "value": "versuch", "fine": 1, "msg": "Die Attribute und ihre Reihenfolge sollen bei der Datenbanktabelle und der Ergebnistabelle gleich sein" },
            { "type": "attributeName", "index": 2, "value": "sprungweite", "fine": 1, "msg": "Die Attribute und ihre Reihenfolge sollen bei der Datenbanktabelle und der Ergebnistabelle gleich sein" },
            { "type": "sameResult", "value": "SELECT * FROM WEITSPRUNG WHERE versuch = 2", "fine": 1 },
            { "type": "accept", "unlock": true }
        ]
    },
    {
        "task": "<p>Ob ein Attribut, welches in der Auswahlbedingung vorkommt, in die Ergebnistabelle übernommen wird oder nicht, das spielt für die Auswahlbedingung keine Rolle. Auch folgende Aufgabe ist lösbar:</p><p>Schreibe und teste eine Abfrage, die zu von allen Datensätzen des letzten Versuchs nur erst <code>name</code> und dann <code>sprungweite</code> liefert.</p>",
        "keepCode": true,
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE versuch = 2",
        "tables": TABLES,
        "locked": true,
        "checkCode": [
            { "type": "compareQuery", "value": "'3'", "substring": true, "inverted": true, "fine": 1, "msg": "Anführungszeichen brauchst du nur bei Zeichenketten, nicht bei Zahlen!" },
            { "type": "keyword", "value": "as", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AS“ nicht erlaubt…" },
            { "type": "keyword", "value": "and", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AND“ nicht erlaubt…" },
            { "type": "keyword", "value": "or", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „OR“ nicht erlaubt…" },
            { "type": "keyword", "value": "not", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „NOT“ nicht erlaubt…" },
            { "type": "keyword", "value": "order", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „ORDER BY“ nicht erlaubt…" }
        ],
        "checkTermination": [
            { "type": "attributeCount", "value": 2, "fine": 1, "msg": "Nur „name“ und „sprungweite“ sollen in der Ergebnistabelle stehen" },
            { "type": "attributeName", "index": 0, "value": "name", "fine": 1, "msg": "Das erste Attribut der Ergebnistabelle soll „name“ sein" },
            { "type": "attributeName", "index": 1, "value": "sprungweite", "fine": 1, "msg": "Das zweite Attribut der Ergebnistabelle soll „sprungweite“ sein" },
            { "type": "sameResult", "value": "SELECT name, sprungweite FROM WEITSPRUNG WHERE versuch = 3", "fine": 1 },
            { "type": "accept", "unlock": true }
        ]
    },
    {
        "task": "<p>Schreibe und teste nun eine Abfrage, die nur die Namen aller liefert, die übertreten haben.</p>",
        "keepCode": true,
        "code": "SELECT name, sprungweite\n    FROM WEITSPRUNG\n    WHERE versuch = 3",
        "tables": TABLES,
        "locked": true,
        "checkCode": [
            { "type": "compareQuery", "value": "'0'", "substring": true, "inverted": true, "fine": 1, "msg": "Anführungszeichen brauchst du nur bei Zeichenketten, nicht bei Zahlen!" },
            { "type": "keyword", "value": "as", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AS“ nicht erlaubt…" },
            { "type": "keyword", "value": "and", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „AND“ nicht erlaubt…" },
            { "type": "keyword", "value": "or", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „OR“ nicht erlaubt…" },
            { "type": "keyword", "value": "not", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „NOT“ nicht erlaubt…" },
            { "type": "keyword", "value": "order", "max": 0, "fine": 0, "msg": "In dieser Aufgabe ist „ORDER BY“ nicht erlaubt…" }
        ],
        "checkTermination": [
            { "type": "attributeCount", "value": 1, "fine": 1, "msg": "Nur die Namen sollen in die Ergebnistabelle" },
            { "type": "attributeName", "index": 0, "value": "name", "fine": 1, "msg": "Der Name soll in der Ergebnistabelle stehen" },
            { "type": "and", "value": [
                { "type": "sameResult", "value": "SELECT name FROM WEITSPRUNG WHERE sprungweite = 0" },
                { "type": "sameResult", "value": "SELECT DISTINCT name FROM WEITSPRUNG WHERE sprungweite = 0" }
            ], "fine": 1 },
            { "type": "accept", "unlock": true }
        ]
    },
    {
        "task": "<p>Wir hatten bislang stets Auswahlbedingungen der Form „Attribut gleich fester Wert“. Aber auch andere Gleichungen können die Auswahlbedingung sein. Auch das folgende Beispiel ist eine gültige Abfrage.</p><p>Schaue dir zuerst diese SQL-Abfrage nur an, ohne sie auszuprobieren und beantworte die Frage.</p>",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE versuch * 2 = sprungweite",
        "readonly": true,
        "tables": TABLES,
        "locked": false,
        "checkCode": [
            { "type": "accept", "accept": false, "fine": 1, "msg": "Du sollst die Abfrage noch nicht ausprobieren!" }
        ],
        "checkTermination": [],
        "questionWrongFine": 1,
        "question": {
            "html": "Welche Datensätze übernimmt die gezeigte Abfrage in die Ergebnistabelle?",
            "choices": [
                { "html": "Nur Datensätze, bei denen die Sprungweite ", "type": "number", "accept": 2, "unit": "-mal so groß wie die Nummer des Versuchs ist", "correct": true },
                { "html": "Alle" },
                { "html": "Keine" },
                { "html": "Nur Datensätze mit Sprungweite ", "type": "number", "min": 0, "max": 10, "step": 0.01, "unit": " m" },
                { "html": "Nur Datensätze zum Versuch ", "type": "number", "min": 1, "max": 3, "step": 1 },
                { "html": "Nur Datensätze von ", "type": "text" }
            ]
        }
    },
    {
        "task": "Probiere nun diese SQL-Abfrage aus.",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE versuch * 2 = sprungweite",
        "readonly": true,
        "tables": TABLES,
        "locked": true,
        "checkCode": [
        ],
        "checkTermination": [
            { "type": "accept", "unlock": true }
        ]
    },
    {
        "task": "Auch das ist eine gültige SQL-Abfrage. Beantworte die Frage, ohne die Abfrage auszuführen.",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE 2 = 2",
        "readonly": true,
        "tables": TABLES,
        "locked": false,
        "checkCode": [
            { "type": "accept", "accept": false, "fine": 1, "msg": "Du sollst die Abfrage noch nicht ausprobieren!" }
        ],
        "checkTermination": [],
        "questionWrongFine": 1,
        "question": {
            "html": "Welche Datensätze übernimmt die gezeigte Abfrage in die Ergebnistabelle?",
            "response": "Diese Auswahlbedingung bzw. Gleichung ist für alle Datensätze wahr, weil sie unabhängig von Attributwerten richtig ist.",
            "choices": [
                { "html": "Nur Datensätze, bei denen die Sprungweite ", "type": "number", "unit": "-mal so groß wie die Nummer des Versuchs ist" },
                { "html": "Alle", "correct": true },
                { "html": "Keine" },
                { "html": "Nur Datensätze mit Sprungweite ", "type": "number", "min": 0, "max": 10, "step": 0.01, "unit": " m" },
                { "html": "Nur Datensätze zum Versuch ", "type": "number", "min": 1, "max": 3, "step": 1 },
                { "html": "Nur Datensätze von ", "type": "text" }
            ]
        }
    },
    {
        "task": "Probiere nun diese SQL-Abfrage aus.",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE 2 = 2",
        "readonly": true,
        "tables": TABLES,
        "locked": true,
        "checkCode": [
        ],
        "checkTermination": [
            { "type": "accept", "unlock": true }
        ]
    },
    {
        "task": "Auch das ist eine gültige SQL-Abfrage. Beantworte die Frage, ohne die Abfrage auszuführen.",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE 'Zwei' = '2'",
        "readonly": true,
        "tables": TABLES,
        "locked": false,
        "checkCode": [
            { "type": "accept", "accept": false, "fine": 1, "msg": "Du sollst die Abfrage noch nicht ausprobieren!" }
        ],
        "checkTermination": [],
        "questionWrongFine": 1,
        "question": {
            "html": "Welche Datensätze übernimmt die gezeigte Abfrage in die Ergebnistabelle?",
            "response": "Diese Auswahlbedingung bzw. Gleichung ist für keinen Datensatz wahr, weil sie unabhängig von Attributwerten niemals korrekt ist. Denn die beiden Zeichenketten sind verschieden.",
            "choices": [
                { "html": "Nur Datensätze, bei denen die Sprungweite ", "type": "number", "unit": "-mal so groß wie die Nummer des Versuchs ist" },
                { "html": "Alle" },
                { "html": "Keine", "correct": true },
                { "html": "Nur Datensätze mit Sprungweite ", "type": "number", "min": 0, "max": 10, "step": 0.01, "unit": " m" },
                { "html": "Nur Datensätze zum Versuch ", "type": "number", "min": 1, "max": 3, "step": 1 },
                { "html": "Nur Datensätze von ", "type": "text" }
            ]
        }
    },
    {
        "task": "Probiere nun diese SQL-Abfrage aus.",
        "code": "SELECT *\n    FROM WEITSPRUNG\n    WHERE 'Zwei' = '2'",
        "readonly": true,
        "tables": TABLES,
        "locked": true,
        "checkCode": [
        ],
        "checkTermination": [
            { "type": "accept", "unlock": true }
        ]
    }
];

window.sqlCodingTask = {
    levels: LEVELS
};
})(window);
